@@ -18,9 +18,9 @@
 #define myx_public_interface_h
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
-#define __LCC__
+#include <winsock2.h>
 #endif
- 
+
 #include <mysql.h>
 #include <myx_aux_functions.h>
 #include <stdio.h>
@@ -1048,7 +1048,7 @@ MYX_PUBLIC_FUNC int myx_get_mysql_major_version(MYSQL *mysql);
 MYX_PUBLIC_FUNC int myx_get_mysql_minor_version(MYSQL *mysql);
 
 MYX_PUBLIC_FUNC char * myx_get_server_variable(MYSQL *mysql, const char *name);
-MYX_PUBLIC_FUNC const char * myx_get_server_charset_name(MYSQL *mysql);
+/*MYX_PUBLIC_FUNC const char * myx_get_server_charset_name(MYSQL *mysql);*/
 
 MYX_PUBLIC_FUNC char * myx_convert_dbstr_utf8(MYSQL *mysql, const char* str);
 MYX_PUBLIC_FUNC char * myx_convert_utf8_dbstr(MYSQL *mysql, const char* str);
@@ -1106,7 +1106,7 @@ MYX_PUBLIC_FUNC const char * myx_tg(MYX_TRANS *trans, const char *id_textgroup,
 
 //Various
 //Check if a string matches a given search string
-MYX_PUBLIC_FUNC int myx_match_string(const char *text, const char *search, int case_sensitive);
+MYX_PUBLIC_FUNC int myx_match_pattern(const char *text, const char *search, int case_sensitive);
 
 //Check whether a given string can be properly converted to an encoding
 MYX_PUBLIC_FUNC int myx_check_utf8_convertible(const char *str, const char *encoding);
