@@ -46,6 +46,12 @@
 # include <unistd.h>
 #endif
 
+// MacOS X
+#if defined(__APPLE__) && defined(__MACH__)
+#include <sys/sysctl.h>
+#include <mach/machine.h>
+#endif
+
 #include "myx_aux_functions.h"
 
 
@@ -511,6 +517,57 @@ char *strcpy2(char *dst, const char *src)
   return dst;
 }
 
+char *utf8_str_trim(char *str)
+{
+  size_t len= strlen(str);
+  size_t new_len;
+  gchar * start, * finish;
+
+  // Get length of leading spaces
+  gchar * pos= str;
+  gchar * end= str + len;
+  gunichar symbol;
+  for ( symbol= g_utf8_get_char(pos);  
+        g_unichar_isspace(symbol);
+        symbol= g_utf8_get_char(pos))
+  {
+    pos= g_utf8_next_char(pos);
+    if (pos==end)
+      break;
+  }
+  start= pos;
+
+  //Get length of trailing spaces
+  pos= end;
+//XXX
+  // the following line and (g_utf8_next_char()) won't do anything
+  // whoever wrote it, please check what's really intended and fix (or remove)
+  g_utf8_prev_char(pos);
+  for ( symbol= g_utf8_get_char(pos);  
+        g_unichar_isspace(symbol);
+        symbol= g_utf8_get_char(pos))
+  {
+    pos= g_utf8_find_prev_char(start,pos);
+    pos= g_utf8_prev_char(pos);
+    if (pos==start)
+      break;
+  }
+  g_utf8_next_char(pos);
+  finish= pos;
+  new_len= finish-start;
+
+  if (start!=end || new_len!=len)
+  {
+    //shift chars to the begining of the string
+    memmove(str, start, new_len);
+
+    //terminate string
+    str[new_len]=0;
+  }
+
+  return str;
+}
+
 char *str_trim(char *str)
 {
   SIZE_T i, j;
@@ -684,15 +741,20 @@ int set_value(char **string_list, unsigned int string_list_num, char *name, char
     if(strncmp(string_list[i], name, name_len)==0 && 
        (string_list[i][name_len]=='\0' || string_list[i][name_len]=='='))
     {
-      string_list[i]= g_realloc(string_list[i], (name_len+strlen(new_value)+2));
 
-      if (string_list[i][name_len]=='=') 
+      if (!new_value)
       {
-        strcpy(string_list[i]+name_len+1, new_value);
+        g_free(string_list[i]);
+        if (string_list_num > 0)
+          memmove(string_list+i, string_list+i+1, string_list_num-1);
       }
       else
       {
-        sprintf(string_list[i], "%s=%s", string_list[i], new_value);
+        string_list[i]= g_realloc(string_list[i], (name_len+strlen(new_value)+2));
+        if (string_list[i][name_len]=='=') 
+          strcpy(string_list[i]+name_len+1, new_value);
+        else
+          sprintf(string_list[i], "%s=%s", string_list[i], new_value);
       }
 
       rc= 0;
@@ -913,45 +975,142 @@ char *get_local_os_name()
 }
 
 
-char *get_local_hardware_info()
+#if defined(__APPLE__) && defined(__MACH__)
+static const char *get_cpu_type_name(int cpu_type, int cpu_subtype)
+{
+  switch (cpu_type)
+  {
+    case CPU_TYPE_I386:
+      return "ix86";
+      break;
+    case CPU_TYPE_POWERPC:
+      switch (cpu_subtype)
+      {
+        case CPU_SUBTYPE_POWERPC_601:
+          return "PowerPC 601";
+        case CPU_SUBTYPE_POWERPC_602:
+          return "PowerPC 602";
+        case CPU_SUBTYPE_POWERPC_603:
+          return "PowerPC 603";
+        case CPU_SUBTYPE_POWERPC_603e:
+          return "PowerPC 603e";
+        case CPU_SUBTYPE_POWERPC_603ev:
+          return "PowerPC 603ev";
+        case CPU_SUBTYPE_POWERPC_604:
+          return "PowerPC 604";
+        case CPU_SUBTYPE_POWERPC_604e:
+          return "PowerPC 604e";
+        case CPU_SUBTYPE_POWERPC_620:
+          return "PowerPC 620";
+        case CPU_SUBTYPE_POWERPC_750:
+          return "PowerPC G3";
+        case CPU_SUBTYPE_POWERPC_7400:
+          return "PowerPC G4";
+        case CPU_SUBTYPE_POWERPC_7450:
+          return "PowerPC G4";
+        case CPU_SUBTYPE_POWERPC_970:
+          return "PowerPC G5";
+      }
+      break;
+  }
+  return "Unknown";
+}
+
+
+// MacOS X
+static int _get_hardware_info(char **cpu, char **clock, int *cpu_count, unsigned long *mem_kb)
+{
+  int mib[2];
+  size_t length;
+  int tmp;
+  uint64_t memsize;
+  char *mclass;
+  int cpu_type, cpu_subtype;
+  
+  // get machine class
+  mib[0]= CTL_HW;
+  mib[1]= HW_MACHINE;
+  sysctl(mib, 2, NULL, &length, NULL, 0);
+  mclass= g_malloc(length*sizeof(char*));
+  sysctl(mib, 2, mclass, &length, NULL, 0);
+
+  // get machine arch
+  length= sizeof(cpu_type);
+  sysctlbyname("hw.cputype", &cpu_type, &length, NULL, 0);
+  length= sizeof(cpu_subtype);
+  sysctlbyname("hw.cpusubtype", &cpu_subtype, &length, NULL, 0);
+  
+  *cpu= g_strdup_printf("%s (%s)", mclass, get_cpu_type_name(cpu_type, cpu_subtype));
+  g_free(mclass);
+  
+  // get cpu clock
+  mib[0]= CTL_HW;
+  mib[1]= HW_CPU_FREQ;
+  length= sizeof(tmp);
+  if (sysctl(mib, 2, &tmp, &length, NULL, 0) < 0)
+    *clock= g_strdup_printf("?");  
+  else
+    *clock= g_strdup_printf("%.01f", (double)tmp/1000000.0);
+
+  // get cpu count
+  mib[0]= CTL_HW;
+  mib[1]= HW_NCPU;
+  length= sizeof(cpu_count);
+  if (sysctl(mib, 2, cpu_count, &length, NULL, 0) < 0)
+    cpu_count= 1;
+
+  // get memory size
+  mib[0]= CTL_HW;
+  mib[1]= HW_MEMSIZE;
+  length= sizeof(memsize);
+  if (sysctl(mib, 2, &memsize, &length, NULL, 0) < 0)
+  {
+    mib[0]= CTL_HW;
+    mib[1]= HW_PHYSMEM;
+    length= sizeof(tmp);
+    sysctl(mib, 2, &tmp, &length, NULL, 0);
+    *mem_kb= tmp/1024;
+  }
+  else
+    *mem_kb= memsize/1024;
+  
+  return 0;
+}
+
+#else
+// Linux
+static int _get_hardware_info(char **cpu, char **clock, int *cpu_count, unsigned long *mem_kb)
 {
-  char *hardware_string;
-  int processor_count = 0;
-  char processor_name[256];
-  char processor_mhz[128];
-  char total_phys_ram[64];
-  long total_phys_ram_val = 0;
   FILE *proc;
   char line[256];
 
-  
- 
   // fetch processor info from /proc/cpuinfo
-
+  
   proc= fopen("/proc/cpuinfo", "r");
   if (!proc) 
   {
-    return NULL;
+    return -1;
   }
-    
+
+  *cpu_count= 0;
   while (!feof(proc)) 
   {
     if (!fgets(line, sizeof(line), proc))
-        break;
-
+      break;
+    
     if (str_beginswith(line,"model name")) 
     {
-      processor_count++;
-      str_trim(strcpy(processor_name, strchr(line, ':')+1));
+      (*cpu_count)++;
+      *cpu= g_strdup(str_trim(strchr(line, ':')+1));
     } 
     else if (str_beginswith(line,"cpu MHz"))
     {
-      str_trim(strcpy(processor_mhz, strchr(line, ':')+1));
+      *clock= g_strdup(str_trim(strchr(line, ':')+1));
     }
   }
   fclose(proc);
   
-
+  
   // fetch physical memory info from /proc/meminfo
   proc= fopen("/proc/meminfo", "r");
   if (proc) 
@@ -966,33 +1125,57 @@ char *get_local_hardware_info()
     while (*ptr && *ptr==' ') ptr++;
     end= strchr(ptr, ' ');
     if (end)
-        *end= 0;
-    total_phys_ram_val = atoi(str_trim(ptr));
+      *end= 0;
+    
+    if (strstr(end, "mB") || strstr(end, "MB"))
+      *mem_kb= strtoul(str_trim(ptr), NULL, 10)*1024;
+    else if (strstr(end, "kB") || strstr(end, "KB"))
+      *mem_kb= strtoul(str_trim(ptr), NULL, 10);
+    else
+      *mem_kb= strtoul(str_trim(ptr), NULL, 10)/1024;
   }
+  return 0;
+}
+#endif
 
-  if(total_phys_ram_val>=1072410624/1.9)
+
+char *get_local_hardware_info()
+{
+  char *hardware_string;
+  int processor_count = 0;
+  char *processor_name= NULL;
+  char *processor_mhz= NULL;
+  char total_phys_ram[64];
+  unsigned long total_phys_ram_val = 0;
+
+  
+  _get_hardware_info(&processor_name, &processor_mhz, &processor_count,
+                     &total_phys_ram_val);
+  
+  if (total_phys_ram_val>=(1024*1024)/1.9)
   {
-    sprintf(total_phys_ram, "%1.1f GB RAM", (double)total_phys_ram_val/1072410624.0);
+    sprintf(total_phys_ram, "%1.1f GB RAM", (double)total_phys_ram_val/(1024*1024));
   }
-  else if(total_phys_ram_val>=1047276/1.9)
+  else if(total_phys_ram_val>=1024/1.9)
   {
-    sprintf(total_phys_ram, "%1.0f MB RAM", (double)total_phys_ram_val/1047276.0);
+    sprintf(total_phys_ram, "%1.0f MB RAM", (double)total_phys_ram_val/1024);
   }
   else
   {
-    sprintf(total_phys_ram, "%ld B RAM", total_phys_ram_val);
+    sprintf(total_phys_ram, "%ld KB RAM", total_phys_ram_val);
   }
 
-  hardware_string= g_malloc(16+strlen(processor_name)+strlen(processor_mhz)+strlen(total_phys_ram));
-
   if(processor_count>1)
   {
-    sprintf(hardware_string, "%dx %s %s MHz, %s", processor_count, processor_name, processor_mhz, total_phys_ram);
+    hardware_string= g_strdup_printf("%dx %s %s MHz, %s", processor_count, processor_name, processor_mhz, total_phys_ram);
   }
   else
   {
-    sprintf(hardware_string, "%s %s MHz, %s", processor_name, processor_mhz, total_phys_ram);
+    hardware_string= g_strdup_printf("%s %s MHz, %s", processor_name, processor_mhz, total_phys_ram);
   }
+  
+  g_free(processor_name);
+  g_free(processor_mhz);
 
   return hardware_string;
 }
@@ -1418,7 +1601,7 @@ double timer_stop(MYX_TIMER_VALUE *tval)
 #endif
 
 
-void *vec_insert_resize(void *vec, gulong size, gulong *vecsize, guint pos, void *data)
+void *vec_insert_resize(void *vec, guint size, guint *vecsize, guint pos, void *data)
 {
   vec= g_realloc(vec, size*(*vecsize+1));
 
@@ -1433,7 +1616,7 @@ void *vec_insert_resize(void *vec, gulong size, gulong *vecsize, guint pos, void
 }
 
 
-void *vec_remove(void *vec, gulong size, gulong *vecsize, guint pos)
+void *vec_remove(void *vec, guint size, guint *vecsize, guint pos)
 {
   if (*vecsize > 0 && ((size_t)pos) < (*vecsize-1))
     memmove((char*)vec+pos*size, (char*)vec+(pos+1)*size, size*(*vecsize-pos-1));
@@ -1646,3 +1829,17 @@ int myx_mkdir(const char *filename, int mode)
 #endif
 }
 
+int get_str_index(char **string_list, unsigned int string_list_num, const char *search)
+{
+  unsigned int i= 0, index= -1;
+
+  while (i<string_list_num)
+  {
+    if (strcmp2(string_list[i], search) == 0)
+      return i;
+
+    i++;
+  }
+
+  return i;
+}
