@@ -18,6 +18,9 @@
 #include <string.h>
 #include <stdlib.h>
 #include <assert.h>
+#ifdef __GNUC__
+#include <unistd.h>
+#endif
 
 #include "myx_library.h"
 #include "myx_network.h"
@@ -45,9 +48,6 @@ typedef struct {
   short major_version;
   short patchlevel;
   char quote_char;
-  char pad0;
-  int pad1;
-  int pad2;
 } MYX_MYSQL;
 
 
@@ -337,7 +337,7 @@ int myx_set_my_cnf_path(const char *path)
 MYX_MYSQL *myx_mysql_get_private(MYSQL *mysql)
 {
   if (mysql)
-    return (MYX_MYSQL*)((char*)mysql - sizeof(MYX_MYSQL));
+    return (MYX_MYSQL*)((char*)mysql + sizeof(MYSQL));
   return NULL;
 }
 
@@ -354,10 +354,10 @@ MYX_MYSQL *myx_mysql_get_private(MYSQL *mysql)
 MYSQL* myx_mysql_init()
 {
   char *data;
-  data= g_malloc0(sizeof(MYX_MYSQL)+sizeof(MYSQL));
-  if (data) 
+  data= g_malloc0(sizeof(MYSQL)+sizeof(MYX_MYSQL));
+  if (data)
   {
-    MYSQL *mysql= (MYSQL*)(data + sizeof(MYX_MYSQL));
+    MYSQL *mysql= (MYSQL*)data;
 
     mysql_init(mysql);
   
@@ -407,7 +407,8 @@ void myx_mysql_set_query_hooks(MYSQL *mysql, MYX_QUERY_CALLBACK pre_hook,
 int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
 {
   MYSQL_REAL_CONNECT_PARAMETERS connect_options= { mysql, 0, 0};  
-
+  char *unix_socket= NULL;
+  
   /* setup advanced options */
   char **advanced_option= user_conn->advanced_options;
   char **end_advanced_option= advanced_option+user_conn->advanced_options_num;
@@ -427,23 +428,49 @@ int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
     }
   }
 
+#if !(defined(__WIN__) || defined(_WIN32) || defined(_WIN64))
+  if (!connect_options.unix_socket)
+  {
+    FILE *f= popen("mysql_config --socket", "r");
+    if (f)
+    {
+      char buffer[1024];
+      
+      if (fgets(buffer, sizeof(buffer), f))
+      {
+        char *ptr= strchr(buffer, '\n');
+        if (ptr)
+        {
+          *ptr= 0;
+          unix_socket= g_strdup(buffer);
+        }
+      }
+      pclose(f);
+    }
+  }
+#endif
+  
   //This stmt tells the server that the parameters of
   //of mysql_real_connect are in utf8
   //mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8");
 
   if (!mysql_real_connect(mysql, user_conn->hostname, user_conn->username,
                           user_conn->password, user_conn->schema,
-                          user_conn->port, connect_options.unix_socket,
+                          user_conn->port, 
+                          unix_socket ? unix_socket : connect_options.unix_socket,
                           connect_options.client_flag | EXTRA_CONNECT_FLAGS))
   {
+    g_free(unix_socket);
     return -1;
   }
 
+  g_free(unix_socket);
+
   // set a high timeout value
   mysql_query(mysql, "SET SESSION interactive_timeout=1000000");
 
   // make sure we're not in ansi_quotes mode, unless asked to
-  if (mysql_version_is_later_or_equal_than(mysql,4,0))
+  if (mysql_version_is_later_or_equal_than(mysql,4,1))
   {
     if (connect_options.ansi_quotes)
       mysql_query(mysql, "SET SESSION sql_mode='ANSI_QUOTES'");
@@ -489,12 +516,9 @@ int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
 *//////////////////////////////////////////////////////////////////////////////
 int myx_mysql_close(MYSQL *mysql)
 {
-  MYX_MYSQL *priv;
-  
   mysql_close(mysql);
   
-  priv= myx_mysql_get_private(mysql);
-  g_free(priv);
+  g_free(mysql);
 
   return 0;
 }
@@ -742,9 +766,6 @@ MYX_MACHINE_INFO * myx_get_server_info(MYX_USER_CONNECTION *user_conn,
   return machine_info;
 }
 
-#ifdef __GNUC__
-#include <unistd.h>
-#endif
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief get information about the client machine
     @ingroup Host_information_management
@@ -804,10 +825,9 @@ int myx_free_pc_info(MYX_MACHINE_INFO *machine_info)
 }
 
 
-//#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 /*
  *----------------------------------------------------------------------
- * myx_match_string
+ * myx_match_pattern
  *
  * SYNOPSIS
  *   Match a string to a search pattern.
@@ -830,23 +850,22 @@ int myx_free_pc_info(MYX_MACHINE_INFO *machine_info)
  *
  *----------------------------------------------------------------------
  */
-static gchar dummy(gchar c)
+static gunichar dummy(gunichar c)
 {
   return c;
 }
 
-int myx_match_string(const char *text, const char *search, int case_sensitive)
+int myx_match_pattern(const char *text, const char *search, int case_sensitive)
 {
-#define MANY '%'
-#define SINGLE '_'
-
 #ifdef __GNUC__
   const char *stack[strlen(text)+2];
 #else
   const char **stack= g_malloc(sizeof(char)*((unsigned int)strlen(text)+2));
 #endif
   int stackp= 0;
-  gchar (*normcase)(gchar)= case_sensitive ? dummy : g_ascii_toupper;
+  const char *tend= text+g_utf8_strlen(text, (gssize)strlen(text));
+  const char *send= search+g_utf8_strlen(text, (gssize)strlen(search));
+  gunichar (*normcase)(gunichar)= case_sensitive ? dummy : g_unichar_toupper;
 
   if (!search || !*search)
     return -1;
@@ -855,24 +874,24 @@ int myx_match_string(const char *text, const char *search, int case_sensitive)
   {
     switch (*search)
     {
-     case MANY:
-      ++search;
-      if (*search != '_')
+     case '*':
+      search= g_utf8_next_char(search);
+      if (*search != '?')
       {
-        gchar norm_search= normcase(*search);
+        gunichar norm_search= normcase(*search);
         stack[stackp++]= search;
-        while (*text && *search && normcase(*text) != norm_search)
-          ++text;
+        while (*text && *search && normcase(g_utf8_get_char_validated(text, (gssize)(tend-text))) != norm_search)
+          text= g_utf8_next_char(text);
         break;
       }
 
-     case SINGLE:
-      ++search;
-      ++text;
+     case '?':
+      search= g_utf8_next_char(search);
+      text= g_utf8_next_char(text);
       break;
       
      default:
-      if (normcase(*text) != normcase(*search))
+      if (normcase(g_utf8_get_char_validated(text, (gssize)(tend-search))) != normcase(g_utf8_get_char_validated(search, (gssize)(send-search))))
       {
         if (stackp==0)
         {
@@ -882,14 +901,14 @@ int myx_match_string(const char *text, const char *search, int case_sensitive)
         {
           // backtrack
           search= stack[--stackp];
-          while (*text && *search && normcase(*text) != normcase(*search))
-            ++text;
+          while (*text && *search && normcase(g_utf8_get_char_validated(text, (gssize)(tend-text))) != normcase(g_utf8_get_char_validated(search, (gssize)(send-search))))
+            text= g_utf8_next_char(text);
         }
       }
       else
       {
-        ++search;
-        ++text;
+        search= g_utf8_next_char(search);
+        text= g_utf8_next_char(text);
       }
       break;
     }
@@ -900,7 +919,6 @@ int myx_match_string(const char *text, const char *search, int case_sensitive)
 #endif
   return !*search;
 }
-//#endif
 
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief get charset name of server
@@ -1066,7 +1084,7 @@ char *myx_convert_utf8_dbstr_with_length(MYSQL *mysql, const char* str,
     //Get server charset
     res= g_convert(str, str_length,
                    iconv_char_name(myx_get_server_charset_name(mysql)),
-                   "UTF-8", &bytes_read, new_length, &error);
+                   "UTF-8", &bytes_read, (gsize*)new_length, &error);
 
     //check if it worked
     if (!error)
@@ -1267,7 +1285,7 @@ int myx_check_utf8_convertible(const char *str, const char *encoding)
 {
   GIConv ic= g_iconv_open(encoding, "utf-8");
   char buffer[256];
-  int in_left, out_left, count;
+  gsize in_left, out_left, count;
   char *in, *out;
   if (!ic)
   {
