@@ -117,7 +117,7 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
   MYSQL_RES *res;
   MYSQL_ROW row;
   MYSQL_FIELD *fields;
-  int r;
+  int r= 0;
   unsigned long *field_lengths;
   unsigned int i, k;
   unsigned long row_count, rows_allocated= 0;
@@ -168,9 +168,10 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
 
   if (sql)
     r= myx_mysql_query(mysql, resultset->query->sql);
+#if MYSQL_VERSION_ID >= 50000
   else
     r= mysql_next_result(mysql);
-
+#endif
   // Stop query timer
   resultset->query_time= timer_stop(&timer);
 
@@ -360,8 +361,11 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
     return NULL;
   }
 
+#if MYSQL_VERSION_ID >= 50000
   resultset->has_more= mysql_more_results(mysql);
-  
+#else
+  resultset->has_more= 0;
+#endif
   return resultset;
 }
 
@@ -1443,8 +1447,8 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
       str= NULL;
       for (j= 0; keys_arr[j]; j++)
       {
-        char *tmp= g_strdup_printf("%s%s=%s", j>0?", ":"",keys_arr[j], values_arr[j]);
-        
+        char *tmp= g_strdup_printf("%s`%s`=%s", j>0?", ":"",keys_arr[j], values_arr[j]);
+
         str= str_g_append_and_free(str, tmp);
       }
       g_strfreev(keys_arr);
@@ -1759,38 +1763,43 @@ int myx_query_compare_results(MYX_RESULTSET *ra, MYX_RESULTSET *rb)
     }
     else
     {
-      bigint diff_mask= 0;
+      bigint diff_mask = 0;
       int eq= 1;
-      // both rows exist, compare the contents
+      // Both rows exist, compare the contents, but set the column mask only for the first 60 columns.
+      // 60 instead 64 is because of the lower nibble, which is used for the MYX_RD_DIFFERS flag.
       for (i= 0; i < ra->columns_num; i++)
       {
-        if (ra->rows[indexa[ia]].fields[i].value_length != 
-            rb->rows[indexb[ib]].fields[i].value_length)
+        if (ra->rows[indexa[ia]].fields[i].value_length != rb->rows[indexb[ib]].fields[i].value_length)
         {
-          eq= 0;
-          diff_mask|= (1<<i);
+          eq = 0;
+          if (i < 60)
+            diff_mask |= (1 << i);
         }
-        else if (memcmp(ra->rows[indexa[ia]].fields[i].value,
-                        rb->rows[indexb[ib]].fields[i].value,
-                        ra->rows[indexa[ia]].fields[i].value_length)!=0)
-        {
-          eq= 0;
-          diff_mask|= (1<<i);
-        }
-      }
+        else 
+          if (memcmp(ra->rows[indexa[ia]].fields[i].value, rb->rows[indexb[ib]].fields[i].value, ra->rows[indexa[ia]].fields[i].value_length) != 0)
+          {
+            eq = 0;
+            if (i < 60)
+              diff_mask |= (1 << i);
+          };
+      };
       
       new_rowsa[ni]= ra->rows[indexa[ia]];
       new_rowsb[ni]= rb->rows[indexb[ib]];
 
       if (!eq)
       {
-        new_rowsa[ni].diff= (diff_mask<<4) | MYX_RD_DIFFERS;
-        new_rowsb[ni].diff= (diff_mask<<4) | MYX_RD_DIFFERS;
+        // If the diff mask is 0 at this point then we had at least one difference in a column with index > 59.
+        // Mark all columns < 60 as differing in this case.
+        if (diff_mask == 0)
+          diff_mask = -1;
+        new_rowsa[ni].diff = (diff_mask << 4) | MYX_RD_DIFFERS;
+        new_rowsb[ni].diff = (diff_mask << 4) | MYX_RD_DIFFERS;
       }
       else
       {        
-        new_rowsa[ni].diff= (diff_mask<<4) | MYX_RD_MATCHES;
-        new_rowsb[ni].diff= (diff_mask<<4) | MYX_RD_MATCHES;
+        new_rowsa[ni].diff = MYX_RD_MATCHES;
+        new_rowsb[ni].diff = MYX_RD_MATCHES;
       }
       ni++;
       ia++;
