@@ -27,6 +27,23 @@ typedef struct MYX_GRT_MODULE_LOADER_PRIVATE
   JNIEnv *env;
   jclass grt_class;
   jmethodID grt_call_func;
+
+  jclass java_class, java_method;
+  jmethodID java_class_getName, java_class_getMethods, java_method_getName, java_method_getReturnType;
+
+  jclass java_string, java_integer, java_double;
+  jmethodID java_integer_init, java_integer_intValue, java_double_init, java_double_doubleValue;
+
+  jclass java_grtobject;
+  jmethodID java_grtobject_get_id;
+
+  jclass java_grtlist;
+  jmethodID java_grtlist_size, java_grtlist_getObject, java_grtlist_getContentType, 
+    java_grtlist_getContentStructName;
+
+  jclass java_grthashmap;
+  jmethodID java_grthashmap_getKeys, java_grthashmap_getObject, java_grthashmap_getContentType, 
+    java_grthashmap_getContentStructName;
 } MYX_JAVA_LOADER;
 
 
@@ -43,4 +60,6 @@ typedef struct MYX_GRT_FUNCTION_PRIVATE
 MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path,
                                             MYX_GRT_ERROR *error);
 
+static MYX_GRT_ERROR java_cache_java_objects_and_methods(MYX_JAVA_LOADER *priv);
+
 #endif
