@@ -29,6 +29,10 @@
 #ifdef ENABLE_LUA_MODULES
 #include <lua.h>
 #endif
+#ifdef ENABLE_JAVA_MODULES
+#include <jni.h>
+#include <com_mysql_grt_GrtCallbackNative.h>
+#endif
 
 #ifdef __cplusplus
 extern "C" {
@@ -71,11 +75,16 @@ typedef enum
   MYX_GRT_FUNCTION_CALL_ERROR,
   MYX_GRT_MODULE_INIT_ERROR,
   MYX_GRT_BAD_MODULE,
-  MYX_GRT_UNKNOWN_MODULE_TYPE
+  MYX_GRT_UNKNOWN_MODULE_TYPE,
+
+  MYX_GRT_JAVA_NOT_FOUND,
+  MYX_GRT_JAVA_REGISTRY_CORRUPTED,
+  MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED
 } MYX_GRT_ERROR;
 
 typedef enum 
 {
+  MYX_ANY_VALUE= 0,
   MYX_INT_VALUE= 1,
   MYX_REAL_VALUE,
   MYX_STRING_VALUE,
@@ -91,6 +100,14 @@ typedef enum
   MYX_GRT_FUNCTION_IS_OPTIONAL = 1<<3
 } MYX_GRT_FUNCTION_FLAGS;
 
+
+typedef enum
+{
+  MYX_IT_SMALL,
+  MYX_IT_STANDARD,
+  MYX_IT_MANY_STANDARD
+} MYX_ICON_TYPE;
+
 /*
  * Structs
  */
@@ -117,10 +134,19 @@ typedef struct MYX_GRT_STRUCT_MEMBER
 {
   char *name;
 
+  char *caption;
+  char *desc;
+
+  char *default_value;
+
   MYX_GRT_VALUE_TYPE value_type;
-  char *value_struct_name; /* only if value_type == dict or list or string*/
-  MYX_GRT_VALUE_TYPE value_content_type; /* only if value_type == list */
+  char *struct_name; /* only if value_type == dict or string */
+
+  MYX_GRT_VALUE_TYPE content_type; /* only if value_type == dict or list */
+  char *content_struct_name; /* only if value_type == dict or list*/
+
   unsigned int is_ref; /* only if value_type == string or list*/
+  char *overrides;
 } MYX_GRT_STRUCT_MEMBER;
 
 
@@ -129,6 +155,9 @@ typedef struct MYX_GRT_STRUCT
   char *name;
   char *parent_struct_name;
 
+  char *caption;
+  char *desc;
+
   unsigned int members_num;
   MYX_GRT_STRUCT_MEMBER *members; // list of members
 } MYX_GRT_STRUCT;
@@ -141,8 +170,8 @@ typedef struct MYX_GRT_STRUCTS
 
 typedef struct 
 {
-  MYX_GRT_VALUE_TYPE content_type;
-  char *content_struct_name; /* only if content_type == dict */
+  MYX_GRT_VALUE_TYPE content_type; /* only used if this is a typed list, meaning it can only hold values of the given type */
+  char *content_struct_name; /* only if content_type == dict, used to make sure only dicts of the given struct can be added */
 
   unsigned int items_num;
   struct MYX_GRT_VALUE **items;
@@ -157,6 +186,11 @@ typedef struct
 typedef struct
 {
   char *struct_name;
+  MYX_GRT *grt; // the grt member is only set if this struct has been added to the GRT reference cache
+  char *object_path; // the path is only set if this struct has been added to the GRT reference cache
+
+  MYX_GRT_VALUE_TYPE content_type; /* only used if this is a typed dict, meaning it can only hold values of the given type */
+  char *content_struct_name; /* only if content_type == dict, used to make sure only dicts of the given struct can be added */
 
   unsigned int items_num;
   MYX_GRT_DICT_ITEM *items; // this must be in strcmp() order of name
@@ -239,7 +273,8 @@ typedef enum
   MYX_BUILTIN_MODULE_TYPE,
     MYX_JAVA_MODULE_TYPE,
     MYX_LUA_MODULE_TYPE,
-    MYX_PYTHON_MODULE_TYPE
+    MYX_PYTHON_MODULE_TYPE,
+    MYX_PHP_MODULE_TYPE
 //    MYX_PERL_MODULE_TYPE,
 } MYX_GRT_MODULE_TYPE;
 
@@ -260,13 +295,37 @@ MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_register_module_loader(MYX_GRT *grt, MYX_G
 MYX_PUBLIC_FUNC int myx_grt_scan_for_modules(MYX_GRT *grt, const char *directory, MYX_GRT_ERROR *error);
 MYX_PUBLIC_FUNC int myx_grt_scan_for_structs(MYX_GRT *grt, const char *directory, MYX_GRT_ERROR *error);
 
-MYX_PUBLIC_FUNC void myx_grt_add_msg(MYX_GRT *grt, int msg_type, const char *message, MYX_STRINGLIST *details, int copy_details);
+#ifdef ENABLE_JAVA_MODULES
+JNIEXPORT jstring JNICALL Java_Grt_callGrtFunction
+  (JNIEnv *, jobject, jlong, jstring, jstring, jstring);
+#endif
 
+// ----------------------------------------------------------------------------------------------------------------------
 // Shell
 
 MYX_PUBLIC_FUNC void myx_grt_set_output_callback(MYX_GRT *grt, void *user_data,
                                  void (*process_output_func)(const char *text, void *userdata));
 
+// ----------------------------------------------------------------------------------------------------------------------
+// GRT global functions
+
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_get_root(MYX_GRT *grt);
+MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_set_root(MYX_GRT *grt, MYX_GRT_VALUE *new_root);
+
+MYX_GRT_VALUE * myx_grt_reference_cache_lookup(MYX_GRT *grt, const char *ref_id);
+MYX_PUBLIC_FUNC void myx_grt_reference_cache_clear(MYX_GRT *grt);
+MYX_PUBLIC_FUNC void myx_grt_reference_cache_rescan(MYX_GRT *grt);
+
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_retrieve_from_file(const char *filename);
+MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT_VALUE *value, const char *filename);
+
+MYX_PUBLIC_FUNC MYX_GRT_MSGS * myx_grt_get_msgs(MYX_GRT *grt, unsigned int count);
+MYX_PUBLIC_FUNC void myx_grt_free_msgs(MYX_GRT_MSGS *msgs);
+MYX_PUBLIC_FUNC void myx_grt_add_msg(MYX_GRT *grt, int msg_type, const char *message, MYX_STRINGLIST *details, int copy_details);
+
+MYX_PUBLIC_FUNC char * myx_grt_get_guid();
+
+// ----------------------------------------------------------------------------------------------------------------------
 // Modules
 
 MYX_PUBLIC_FUNC void myx_grt_module_set_log_callback(MYX_GRT *grt, 
@@ -290,7 +349,8 @@ MYX_PUBLIC_FUNC MYX_GRT_FUNCTION * myx_grt_module_function_get_by_index(MYX_GRT_
 MYX_PUBLIC_FUNC char * myx_grt_module_function_get_params(MYX_GRT_FUNCTION *func);
 MYX_PUBLIC_FUNC char * myx_grt_module_function_get_return_type(MYX_GRT_FUNCTION *func);
 
-// GRT Functions
+// ----------------------------------------------------------------------------------------------------------------------
+// GRT module functions
 
 MYX_PUBLIC_FUNC MYX_GRT_FUNCTION * myx_grt_function_get(MYX_GRT *grt, const char *module, const char *function_name, int search_parent);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_function_call(MYX_GRT *grt, MYX_GRT_FUNCTION *func,
@@ -299,143 +359,222 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_function_call(MYX_GRT *grt, MYX_GRT_FUNC
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_function_get_and_call(MYX_GRT *grt, const char *module, const char *function_name, int search_parent,
                                     MYX_GRT_VALUE *argument, MYX_GRT_ERROR *error);
 
-// GRT global functions
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_function_create_result(MYX_GRT_VALUE *result);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_function_create_error_result(const char *error, const char *error_details);
+MYX_PUBLIC_FUNC char * myx_grt_function_check_error(MYX_GRT_VALUE *res, int allow_null_as_result);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_function_extract_value_from_result(MYX_GRT_VALUE *result);
 
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_get_root(MYX_GRT *grt);
-MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_set_root(MYX_GRT *grt, MYX_GRT_VALUE *new_root);
-
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_retrieve_from_file(const char *filename);
-MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT_VALUE *value, const char *filename);
-  
+// ----------------------------------------------------------------------------------------------------------------------
 // GRT Structs
 
-MYX_PUBLIC_FUNC int myx_grt_struct_get_count(MYX_GRT *grt);
-MYX_PUBLIC_FUNC int myx_grt_struct_get_child_count(MYX_GRT *grt, const char *struct_name);
-MYX_PUBLIC_FUNC MYX_GRT_STRUCT * myx_grt_struct_get_by_index(MYX_GRT *grt, unsigned int index);
-MYX_PUBLIC_FUNC MYX_GRT_STRUCT * myx_grt_struct_get_child_by_index(MYX_GRT *grt, const char *struct_name, unsigned int index);
-
-MYX_PUBLIC_FUNC MYX_GRT_STRUCT * myx_grt_struct_get(MYX_GRT *grt, const char *name);
-
-MYX_PUBLIC_FUNC MYX_GRT_STRUCTS * myx_grt_structs_get(MYX_GRT *grt);
-MYX_PUBLIC_FUNC int myx_grt_structs_free(MYX_GRT_STRUCTS *gstructs);
-MYX_PUBLIC_FUNC int myx_grt_struct_free(MYX_GRT_STRUCT *gstruct);
-
+// General
+MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_struct_load_and_register(MYX_GRT *grt, const char *filename);
 MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_struct_register(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct);
 
-MYX_PUBLIC_FUNC int myx_grt_struct_check_inherits_from(MYX_GRT *grt, const char *struct_name, const char *parent_name);
+// Packages
+MYX_PUBLIC_FUNC int myx_grt_package_count(MYX_GRT *grt);
+MYX_PUBLIC_FUNC char * myx_grt_package_by_index(MYX_GRT *grt, unsigned int index);
+MYX_PUBLIC_FUNC MYX_STRINGLIST * myx_grt_packages(MYX_GRT *grt);
+
+MYX_PUBLIC_FUNC int myx_grt_package_struct_count(MYX_GRT *grt, const char *package_name);
+MYX_PUBLIC_FUNC MYX_GRT_STRUCT * myx_grt_package_struct_by_index(MYX_GRT *grt, const char *package_name, unsigned int index);
+
+// Structs
+MYX_PUBLIC_FUNC MYX_GRT_STRUCTS * myx_grt_structs_get(MYX_GRT *grt);
+MYX_PUBLIC_FUNC int myx_grt_structs_free(MYX_GRT_STRUCTS *gstructs);
+
+MYX_PUBLIC_FUNC int myx_grt_struct_get_count(MYX_GRT *grt);
+MYX_PUBLIC_FUNC MYX_GRT_STRUCT * myx_grt_struct_get_by_index(MYX_GRT *grt, unsigned int index);
+
+MYX_PUBLIC_FUNC MYX_GRT_STRUCTS * myx_grt_struct_load_list(const char *filename, MYX_GRT_ERROR *error);
+MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_struct_save_list(MYX_GRT_STRUCTS *gstructs, const char *filename);
+
+// Struct handling
+MYX_PUBLIC_FUNC MYX_GRT_STRUCT * myx_grt_struct_get(MYX_GRT *grt, const char *name);
+MYX_PUBLIC_FUNC int myx_grt_struct_free(MYX_GRT_STRUCT *gstruct);
+
+MYX_PUBLIC_FUNC int myx_grt_struct_get_child_count(MYX_GRT *grt, const char *struct_name);
+MYX_PUBLIC_FUNC MYX_GRT_STRUCT * myx_grt_struct_get_child_by_index(MYX_GRT *grt, const char *struct_name, unsigned int index);
 
 MYX_PUBLIC_FUNC const char * myx_grt_struct_get_name(MYX_GRT_STRUCT *gstruct);
 MYX_PUBLIC_FUNC const char * myx_grt_struct_get_parent_name(MYX_GRT_STRUCT *gstruct);
+MYX_PUBLIC_FUNC const char * myx_grt_struct_get_caption(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct, int *inherited_caption);
+MYX_PUBLIC_FUNC const char * myx_grt_struct_get_desc(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct);
 
+MYX_PUBLIC_FUNC int myx_grt_struct_check_inherits_from(MYX_GRT *grt, const char *struct_name, const char *parent_name);
+
+MYX_PUBLIC_FUNC const char * myx_grt_struct_get_icon(MYX_GRT *grt, const char *source_path, 
+                 MYX_GRT_STRUCT *gstruct, MYX_ICON_TYPE icon_type, unsigned int *length);
+
+// Struct utilites
 MYX_PUBLIC_FUNC char * myx_grt_struct_to_xml(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct, int include_children);
 MYX_PUBLIC_FUNC MYX_GRT_STRUCTS * myx_grt_struct_from_xml(const char *str, size_t size);
 
-MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_struct_save_list(MYX_GRT_STRUCTS *gstructs, const char *filename);
-MYX_PUBLIC_FUNC MYX_GRT_STRUCTS * myx_grt_struct_load_list(const char *filename, MYX_GRT_ERROR *error);
-
-MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_struct_load_and_register(MYX_GRT *grt, const char *filename);
-
 MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_struct_export_java_classes(MYX_GRT_STRUCTS *gstructs, 
                                                                  const char *package_name, const char *output_path);
 
-// GRT Structs Members
+MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_struct_export_php_classes(MYX_GRT_STRUCTS *gstructs, const char *output_path);
 
+
+// GRT Structs Members
 MYX_PUBLIC_FUNC unsigned int myx_grt_struct_get_member_count(MYX_GRT_STRUCT *gstruct);
 MYX_PUBLIC_FUNC MYX_GRT_STRUCT_MEMBER * myx_grt_struct_get_member_by_index(MYX_GRT_STRUCT *gstruct, unsigned int index);
+MYX_PUBLIC_FUNC MYX_GRT_STRUCT_MEMBER * myx_grt_struct_get_member_by_name(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct, const char *name, int check_parent_structs);
+
+MYX_PUBLIC_FUNC int myx_grt_struct_get_member_count_total(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct);
+MYX_PUBLIC_FUNC int myx_grt_struct_get_member_count_total_excluding_struct(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct, const char *excluding_struct_name);
+MYX_PUBLIC_FUNC MYX_GRT_STRUCT_MEMBER * myx_grt_struct_get_member_by_index_total(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct, int index);
 
 MYX_PUBLIC_FUNC const char * myx_grt_struct_get_member_name(MYX_GRT_STRUCT_MEMBER *member);
+MYX_PUBLIC_FUNC const char * myx_grt_struct_get_member_default(MYX_GRT_STRUCT_MEMBER *member);
+MYX_PUBLIC_FUNC const char * myx_grt_struct_get_member_caption(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct, const char *member_name, int check_parent_structs);
+MYX_PUBLIC_FUNC const char * myx_grt_struct_get_member_desc(MYX_GRT_STRUCT_MEMBER *member);
 
-MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_struct_get_member_type(MYX_GRT_STRUCT_MEMBER *member);
-MYX_PUBLIC_FUNC const char * myx_grt_struct_get_member_value_struct_name(MYX_GRT_STRUCT_MEMBER *member);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_struct_member_get_type(MYX_GRT_STRUCT_MEMBER *member);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_struct_member_get_content_type(MYX_GRT_STRUCT_MEMBER *member);
+MYX_PUBLIC_FUNC const char * myx_grt_struct_member_get_struct_name(MYX_GRT_STRUCT_MEMBER *member);
+MYX_PUBLIC_FUNC const char * myx_grt_struct_member_get_content_struct_name(MYX_GRT_STRUCT_MEMBER *member);
+MYX_PUBLIC_FUNC unsigned int myx_grt_struct_member_get_is_ref(MYX_GRT_STRUCT_MEMBER *member);
 
+// ----------------------------------------------------------------------------------------------------------------------
 // GRT Value handling
 
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_retain(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC int myx_grt_value_release(MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC int myx_grt_value_get_current_reference_count(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC int myx_grt_value_print(MYX_GRT *grt, MYX_GRT_VALUE *value);
   
 MYX_PUBLIC_FUNC char * myx_grt_value_to_xml(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_xml(const char *str, size_t size);
 
+MYX_PUBLIC_FUNC char * myx_grt_value_to_xml_global_object(const char *objectPath, MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_xml_global_object(MYX_GRT *grt, const char *str, size_t size);
+
 MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_value_get_type(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC const char * myx_get_value_type_as_string(MYX_GRT_VALUE_TYPE value_type);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_get_value_type_from_string(const char *value_type_name, MYX_GRT_ERROR *error);
+MYX_PUBLIC_FUNC int myx_grt_value_is_simple_type(MYX_GRT_VALUE *value);
 
 MYX_PUBLIC_FUNC int myx_grt_value_as_int(MYX_GRT_VALUE *value);
-MYX_PUBLIC_FUNC double myx_grt_value_as_double(MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC double myx_grt_value_as_real(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC const char * myx_grt_value_as_string(MYX_GRT_VALUE *value);
-MYX_PUBLIC_FUNC char * myx_grt_value_format_as_string(MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC char * myx_grt_value_formated_as_string(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC MYX_GRT_LIST * myx_grt_value_as_list(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC MYX_GRT_DICT * myx_grt_value_as_dict(MYX_GRT_VALUE *value);
 
-// GRT Basic Type Values  
-
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_int(int i);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_double(double d);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_real(double d);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_string(const char *s);
 
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_dup(MYX_GRT_VALUE *value);
+
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_get_by_path(MYX_GRT_VALUE *dict, const char *path);
+MYX_PUBLIC_FUNC int myx_grt_value_set_by_path(MYX_GRT_VALUE *dict, const char *path, MYX_GRT_VALUE *new_value);
+
+
+// ----------------------------------------------------------------------------------------------------------------------
+// GRT List Values
+
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_new(MYX_GRT_VALUE_TYPE content_type, const char *struct_name);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_create_from_stringlist(MYX_STRINGLIST *sl);
+
+MYX_PUBLIC_FUNC MYX_STRINGLIST * myx_grt_list_as_stringlist(MYX_GRT_VALUE *list);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_from_stringlist(MYX_STRINGLIST *str_list);
+
+MYX_PUBLIC_FUNC int myx_grt_list_clear(MYX_GRT_VALUE *list);
+
+// Items
+MYX_PUBLIC_FUNC int myx_grt_list_item_insert(MYX_GRT_VALUE *list, int index, MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC int myx_grt_list_item_add(MYX_GRT_VALUE *list, MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC int myx_grt_list_item_add_as_string(MYX_GRT_VALUE *list, const char *s);
+MYX_PUBLIC_FUNC int myx_grt_list_item_del(MYX_GRT_VALUE *list, int index);
+MYX_PUBLIC_FUNC int myx_grt_list_item_del_as_string(MYX_GRT_VALUE *list, const char *s);
+
+// Items, index based access
+MYX_PUBLIC_FUNC unsigned int myx_grt_list_item_count(MYX_GRT_VALUE *list);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_item_get(MYX_GRT_VALUE *list, unsigned int index);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_item_get_reference_value(MYX_GRT *grt, MYX_GRT_VALUE *list, unsigned int index);
+MYX_PUBLIC_FUNC const char * myx_grt_list_item_get_as_string(MYX_GRT_VALUE *list, unsigned int index);
+MYX_PUBLIC_FUNC int myx_grt_list_set_item(MYX_GRT_VALUE *list, unsigned int index, MYX_GRT_VALUE *new_value);
+
+// Struct/content handling
+MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_list_content_get_type(MYX_GRT_VALUE *list);
+MYX_PUBLIC_FUNC const char * myx_grt_list_content_get_struct_name(MYX_GRT_VALUE *list);
+
+MYX_PUBLIC_FUNC void myx_grt_list_content_set_type(MYX_GRT_VALUE *list, MYX_GRT_VALUE_TYPE content_type);
+MYX_PUBLIC_FUNC int myx_grt_list_content_set_struct_name(MYX_GRT_VALUE *list, const char *struct_name);
+
+// ----------------------------------------------------------------------------------------------------------------------
 // GRT Dictionary Values
 
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_new(const char *struct_name);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_new_typed(MYX_GRT_VALUE_TYPE content_type, const char *content_struct_name);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_create(const char *struct_name, const char *key, ...);
+//MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_create_from_list(MYX_GRT_VALUE *list);
 
-MYX_PUBLIC_FUNC int myx_grt_dict_set_item(MYX_GRT_VALUE *dict, const char *key, MYX_GRT_VALUE *value);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_get_item_value(MYX_GRT_VALUE *dict, const char *key);
-MYX_PUBLIC_FUNC int myx_grt_dict_del_item(MYX_GRT_VALUE *dict, const char *key);
+// Item set functions
+MYX_PUBLIC_FUNC int myx_grt_dict_item_set_value(MYX_GRT_VALUE *dict, const char *key, MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC int myx_grt_dict_item_set_value_from_string(MYX_GRT_VALUE *dict, const char *key, const char *s);
+MYX_PUBLIC_FUNC int myx_grt_dict_item_set_value_from_int(MYX_GRT_VALUE *dict, const char *key, int i);
 
-MYX_PUBLIC_FUNC const char * myx_grt_dict_item_as_string(MYX_GRT_VALUE *dict, const char *key);
-MYX_PUBLIC_FUNC int myx_grt_dict_item_as_int(MYX_GRT_VALUE *dict, const char *key);
-MYX_PUBLIC_FUNC double myx_grt_dict_item_as_double(MYX_GRT_VALUE *dict, const char *key);
+// Item get functions
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_item_get_value(MYX_GRT_VALUE *dict, const char *key);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_item_get_reference_value(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *key);
+MYX_PUBLIC_FUNC const char * myx_grt_dict_item_get_as_string(MYX_GRT_VALUE *dict, const char *key);
+MYX_PUBLIC_FUNC char * myx_grt_dict_item_get_formated_as_string(MYX_GRT_VALUE *dict, const char *key);
+MYX_PUBLIC_FUNC int myx_grt_dict_item_get_as_int(MYX_GRT_VALUE *dict, const char *key);
+MYX_PUBLIC_FUNC double myx_grt_dict_item_get_as_double(MYX_GRT_VALUE *dict, const char *key);
 
-MYX_PUBLIC_FUNC unsigned int myx_grt_dict_item_num(MYX_GRT_VALUE *dict);
+// Item del function
+MYX_PUBLIC_FUNC int myx_grt_dict_item_del(MYX_GRT_VALUE *dict, const char *key);
+
+// Items, index based access
+MYX_PUBLIC_FUNC unsigned int myx_grt_dict_item_count(MYX_GRT_VALUE *dict);
 MYX_PUBLIC_FUNC int myx_grt_dict_item_by_index(MYX_GRT_VALUE *dict, unsigned int index, 
                                           const char **retkey, MYX_GRT_VALUE **retvalue);
 MYX_PUBLIC_FUNC const char * myx_grt_dict_item_key_by_index(MYX_GRT_VALUE *dict, unsigned int index);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_item_value_by_index(MYX_GRT_VALUE *dict, unsigned int index);
-  
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_set_value_by_path(MYX_GRT_VALUE *dict, const char *path);
-MYX_PUBLIC_FUNC int myx_grt_dict_get_value_by_path(MYX_GRT_VALUE *dict, const char *path, MYX_GRT_VALUE *new_value);
 
+// GRT Object function
 MYX_PUBLIC_FUNC const char * myx_grt_dict_name_item_as_string(MYX_GRT_VALUE *dict);
 MYX_PUBLIC_FUNC const char * myx_grt_dict_id_item_as_string(MYX_GRT_VALUE *dict);
 
-// GRT List Values
+// Struct/content handling
+MYX_PUBLIC_FUNC const char * myx_grt_dict_struct_get_name(MYX_GRT_VALUE *dict);
+MYX_PUBLIC_FUNC MYX_GRT_STRUCT * myx_grt_dict_struct_get(MYX_GRT *grt, MYX_GRT_VALUE *dict);
+MYX_PUBLIC_FUNC int myx_grt_dict_struct_set_name(MYX_GRT_VALUE *dict, const char *struct_name);
+MYX_PUBLIC_FUNC int myx_grt_dict_struct_validate(MYX_GRT *grt, MYX_GRT_VALUE *value, const char *struct_name, int strict);
 
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_new(MYX_GRT_VALUE_TYPE content_type, const char *struct_name);
+MYX_PUBLIC_FUNC int myx_grt_dict_struct_inherits_from(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *parent_struct_name);
 
-MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_list_content_type(MYX_GRT_VALUE *list);
-MYX_PUBLIC_FUNC MYX_STRINGLIST * myx_grt_list_as_stringlist(MYX_GRT_VALUE *list);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_dict_content_get_type(MYX_GRT_VALUE *dict);
+MYX_PUBLIC_FUNC const char * myx_grt_dict_content_get_struct_name(MYX_GRT_VALUE *dict);
 
-MYX_PUBLIC_FUNC int myx_grt_list_insert_item(MYX_GRT_VALUE *list, int index, MYX_GRT_VALUE *value);
-#define myx_grt_list_add_item(list, value) myx_grt_list_insert_item(list, -1, value)
-MYX_PUBLIC_FUNC int myx_grt_list_del_item(MYX_GRT_VALUE *list, int index);
-MYX_PUBLIC_FUNC unsigned int myx_grt_list_item_num(MYX_GRT_VALUE *list);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_get_item(MYX_GRT_VALUE *list, unsigned int index);
-MYX_PUBLIC_FUNC int myx_grt_list_set_item(MYX_GRT_VALUE *list, unsigned int index, MYX_GRT_VALUE *new_value);
+MYX_PUBLIC_FUNC void myx_grt_dict_content_set_type(MYX_GRT_VALUE *dict, MYX_GRT_VALUE_TYPE content_type);
 
-// GRT Value - Struct handling
-
-MYX_PUBLIC_FUNC int myx_grt_dict_assign_struct(MYX_GRT_VALUE *dict, const char *struct_name);
-MYX_PUBLIC_FUNC int myx_grt_list_assign_struct(MYX_GRT_VALUE *list, const char *struct_name);
-
-MYX_PUBLIC_FUNC const char * myx_grt_value_struct_name(MYX_GRT_VALUE *value);
-MYX_PUBLIC_FUNC int myx_grt_value_validate_struct(MYX_GRT *grt, MYX_GRT_VALUE *value, const char *struct_name, int strict);
-
-// GRT Utility Value Functions
-
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_make_dict_from_list(MYX_GRT_VALUE *list);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_stringlist(MYX_STRINGLIST *sl);
+MYX_PUBLIC_FUNC const char * myx_grt_dict_get_object_path(MYX_GRT_VALUE *dict);
 
 
+// ----------------------------------------------------------------------------------------------------------------------
 // Java module
+
+#ifdef ENABLE_JAVA_MODULES
 MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_java_init_loader(MYX_GRT *grt, const char *class_path, MYX_GRT_ERROR *error);
+#endif
+
+// ----------------------------------------------------------------------------------------------------------------------
+// PHP module
+
+#ifdef ENABLE_PHP_MODULES
+MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_php_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error);
+#endif
+
+// ----------------------------------------------------------------------------------------------------------------------
+// Lua module
 
 #ifdef ENABLE_LUA_MODULES
-
-// Lua module
 MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_lua_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error);
 
-
 // Lua Shell
 MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_init_lua_shell(MYX_GRT *grt);
 MYX_PUBLIC_FUNC char * myx_grt_lua_shell_get_prompt(MYX_GRT *grt);
@@ -457,9 +596,10 @@ MYX_PUBLIC_FUNC int myx_grt_check_module_connected(MYX_GRT *grt);
 MYX_PUBLIC_FUNC int myx_grt_check_module_message(MYX_GRT *grt, char **message, int timeout);
 MYX_PUBLIC_FUNC int myx_grt_send_module_message(MYX_GRT *grt, const char *message);
   
-//
+// Aux funcs
 MYX_PUBLIC_FUNC int _myx_grt_get_refcount(MYX_GRT_VALUE *value);
 
+
 #ifdef __cplusplus
 }
 #endif
