@@ -29,6 +29,13 @@
 #include "myx_xml_aux_functions.h"
 #include "myx_grt_private.h"
 
+#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+#include <objbase.h>
+#else
+// Linux specific
+#include <uuid/uuid.h>
+#endif
+
 
 static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_by_id);
 static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value, GHashTable *saved_ids);
@@ -71,6 +78,12 @@ MYX_GRT *myx_grt_initialize()
   
   grt->root= myx_grt_dict_new(NULL);
 
+  grt->struct_icon_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
+                                                g_free, g_free);
+
+  grt->reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
+                                                g_free, NULL);
+  
   myx_grt_register_module_loader(grt, myx_builtin_init_loader(grt));
 
 #ifdef remove_this_when_this_gets_used
@@ -179,6 +192,73 @@ MYX_GRT_STRUCTS * myx_grt_structs_get(MYX_GRT *grt)
   return structs;
 }
 
+MYX_STRINGLIST * myx_grt_struct_packages(MYX_GRT *grt)
+{
+  unsigned int i;
+
+  g_return_val_if_fail(grt != NULL, NULL);
+
+  {
+    MYX_STRINGLIST *packages= g_new0(MYX_STRINGLIST, 1);
+
+    for (i= 0; i<grt->structs_num; i++)
+    {
+      MYX_GRT_STRUCT *str= grt->structs + i;
+      char *str_pak= g_strdup(str->name);
+      unsigned int j, l= (unsigned int)strlen(str_pak);
+
+      for (j= l-1; j>0; j--)
+      {
+        if (str_pak[j] == '.')
+          break;
+      }
+      str_pak[j]= 0;
+
+      //See if the package name is already stored
+      for (j= 0; j<packages->strings_num; j++)
+      {
+        if (strcmp3(str_pak, packages->strings[j]) == 0)
+          break;
+      }
+      //If not, add the package name
+      if (j == packages->strings_num)
+      {
+        packages->strings_num++;
+        packages->strings= g_realloc(packages->strings, sizeof(char *)*packages->strings_num);
+        packages->strings[packages->strings_num-1]= g_strdup(str_pak);
+      }
+
+      g_free(str_pak);
+    }
+
+    return packages;
+  }
+}
+
+int myx_grt_package_count(MYX_GRT *grt)
+{
+  unsigned int c;
+  MYX_STRINGLIST *packages= myx_grt_struct_packages(grt);
+
+  c= packages->strings_num;
+
+  myx_free_stringlist(packages);
+
+  return c;
+}
+
+char * myx_grt_package_by_index(MYX_GRT *grt, unsigned int index)
+{
+  char *s;
+  MYX_STRINGLIST *packages= myx_grt_struct_packages(grt);
+
+  s= g_strdup(packages->strings[index]);
+
+  myx_free_stringlist(packages);
+
+  return s;
+}
+
 /**
  ****************************************************************************
  * @brief Returns the number of structs registered in the GRT
@@ -223,6 +303,37 @@ int myx_grt_struct_get_child_count(MYX_GRT *grt, const char *struct_name)
   return c;
 }
 
+int myx_grt_package_struct_count(MYX_GRT *grt, const char *package_name)
+{
+  unsigned int i, c= 0, l= (unsigned int)strlen(package_name);
+
+  g_return_val_if_fail(grt != NULL, 0);
+  g_return_val_if_fail(package_name != NULL, 0);
+
+  for (i= 0; i<grt->structs_num; i++)
+  {
+    MYX_GRT_STRUCT *str= grt->structs + i;   
+
+    if (package_name[0])
+    {
+      char *begin_str= g_strdup_printf("%s.", package_name);
+
+      if (str_beginswith(str->name, begin_str) &&
+          (sub_str_count(".", str->name+l+1) == 0))
+        c++;
+
+      g_free(begin_str);
+    }
+    else
+    {
+      if (sub_str_count(".", str->name) == 0)
+        c++;
+    }
+  }
+
+  return c;
+}
+
 /**
  ****************************************************************************
  * @brief Returns the GRT struct given by an index
@@ -273,6 +384,49 @@ MYX_GRT_STRUCT * myx_grt_struct_get_child_by_index(MYX_GRT *grt, const char *str
   return NULL;
 }
 
+MYX_GRT_STRUCT * myx_grt_package_struct_by_index(MYX_GRT *grt, const char *package_name, unsigned int index)
+{
+  unsigned int i, c= 0, l= (unsigned int)strlen(package_name);
+
+  g_return_val_if_fail(grt != NULL, NULL);
+
+  for (i= 0; i<grt->structs_num; i++)
+  {
+    MYX_GRT_STRUCT *str= grt->structs + i;
+
+    if (package_name[0])
+    {
+      char *begin_str= g_strdup_printf("%s.", package_name);
+
+      if (str_beginswith(str->name, begin_str) &&
+          (sub_str_count(".", str->name+l+1) == 0))
+      {
+        if (c == index)
+        {
+          g_free(begin_str);
+          return str;
+        }
+
+        c++;
+      }
+
+      g_free(begin_str);
+    }
+    else
+    {
+      if (sub_str_count(".", str->name) == 0)
+      {
+        if (c == index)
+          return str;
+
+        c++;
+      }
+    }
+  }
+
+  return NULL;
+}
+
 
 /**
  ****************************************************************************
@@ -314,11 +468,13 @@ MYX_GRT_ERROR myx_grt_set_root(MYX_GRT *grt, MYX_GRT_VALUE *new_root)
   if (myx_grt_value_get_type(new_root) != MYX_DICT_VALUE)
     return MYX_GRT_BAD_VALUE;
 
+  myx_grt_reference_cache_clear(grt);
   myx_grt_value_release(grt->root);
 
   grt->root= new_root;
 
   myx_grt_value_retain(new_root);
+  myx_grt_reference_cache_rescan(grt);
   
   return MYX_GRT_NO_ERROR;
 }
@@ -400,15 +556,16 @@ MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT_VALUE *value, const char *filename)
 }
 
 
-static char *value_type_strings[]=
+/*static char *value_type_strings[]=
 {
-  "int",
+  "all",
+    "int",
     "bigint",
     "real",
     "string",
     "list",
     "dict"
-};
+};*/
 
 
 /**
@@ -468,14 +625,15 @@ static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
   case MYX_LIST_VALUE:
     node= xmlNewTextChild(parent, NULL, "value", NULL);
     xmlNewProp(node, "type", "list");
-    xmlNewProp(node, "content-type", value_type_strings[value->value.l->content_type]);
+    xmlNewProp(node, "content-type", myx_get_value_type_as_string(value->value.l->content_type));
 
     if (value->value.l->content_struct_name)
       xmlNewProp(node, "content-struct-name", value->value.l->content_struct_name);
 
     for (i= 0; i < value->value.l->items_num; i++)
     {
-      g_return_val_if_fail(value->value.l->items[i]->type == value->value.l->content_type, NULL);
+      g_return_val_if_fail((value->value.l->items[i]->type == value->value.l->content_type) ||
+         (value->value.l->content_type == MYX_ANY_VALUE), NULL);
       serialize_to_xml(node, value->value.l->items[i], saved_ids);
     }
     break;
@@ -486,6 +644,12 @@ static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
     if ((value->value.d->struct_name) && (value->value.d->struct_name[0]))
       xmlNewProp(node, "struct-name", value->value.d->struct_name);
 
+    if (value->value.d->content_type != MYX_ANY_VALUE)
+      xmlNewProp(node, "content-type", myx_get_value_type_as_string(value->value.d->content_type));
+
+    if ((value->value.d->content_struct_name) && (value->value.d->content_struct_name[0]))
+      xmlNewProp(node, "content-struct-name", value->value.d->content_struct_name);
+
     if ((id= myx_grt_dict_id_item_as_string(value)))
     {
       if (g_hash_table_lookup(saved_ids, id))
@@ -508,6 +672,8 @@ static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
       xmlNewProp(child, "key", value->value.d->items[i].key);
     }
     break;
+  case MYX_ANY_VALUE:
+    break;
   }
   return node;
 }
@@ -515,7 +681,6 @@ static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
 
 static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_by_id)
 {
-  int i;
   xmlChar *str;
   xmlChar *node_type= xmlGetProp(node, "type");
   MYX_GRT_VALUE *value= NULL;
@@ -535,7 +700,7 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
   else if (strcmp(node_type, "real")==0)
   {
     str= xmlNodeGetContent(node);
-    value= myx_grt_value_from_double(strtod(str, NULL));
+    value= myx_grt_value_from_real(strtod(str, NULL));
     xmlFree(str);
   }
   else if (strcmp(node_type, "string")==0)
@@ -546,7 +711,6 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
   }
   else if (strcmp(node_type, "dict")==0)
   {
-    xmlChar *node_type= xmlGetProp(node, "struct-name");
     xmlChar *link_id= xmlGetProp(node, "link");
     xmlNodePtr child;
 
@@ -570,10 +734,29 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
     }
     else
     {
-      value= myx_grt_dict_new(NULL);
+      xmlChar *prop= xmlGetProp(node, "content-type");
+      MYX_GRT_VALUE_TYPE content_type= MYX_ANY_VALUE;
+      MYX_GRT_ERROR error;
 
-      if (node_type && node_type[0])
-        value->value.d->struct_name= g_strdup(node_type);
+      content_type= myx_get_value_type_from_string(prop, &error);
+      xmlFree(prop);
+
+      if (content_type != MYX_ANY_VALUE)
+      {
+        char *content_struct_name= xmlGetProp(node, "content-struct-name");
+
+        value= myx_grt_dict_new_typed(content_type, content_struct_name);
+
+        xmlFree(content_struct_name);
+      }
+      else
+      {
+        xmlChar *struct_name= xmlGetProp(node, "struct-name");
+
+        value= myx_grt_dict_new(struct_name);
+
+        xmlFree(struct_name);
+      }
 
       child= node->children;
       while (child)
@@ -587,7 +770,7 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
           sub_value= unserialize_from_xml(child, objects_by_id);
           if (sub_value)
           {
-            myx_grt_dict_set_item(value, key, sub_value);
+            myx_grt_dict_item_set_value(value, key, sub_value);
             myx_grt_value_release(sub_value);
           }
           else
@@ -616,42 +799,49 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
     xmlChar *ctype= xmlGetProp(node, "content-type");
     xmlChar *cstruct_name= xmlGetProp(node, "content-struct-name");
     xmlNodePtr child;
+    MYX_GRT_ERROR error;
+    MYX_GRT_VALUE_TYPE content_type= myx_get_value_type_from_string(ctype, &error);
 
-    for (i= 0; sizeof(value_type_strings)/sizeof(char*); i++)
+    /*for (i= 0; sizeof(value_type_strings)/sizeof(char*); i++)
       if (strcmp2(value_type_strings[i], ctype)==0)
       {
         value= myx_grt_list_new(i, NULL);
         break;
-      }
+      }*/
 
-    value->value.l->content_struct_name= g_strdup(cstruct_name);
-
-    child= node->children;
-    while (child)
+    if (error == MYX_GRT_NO_ERROR)
     {
-      MYX_GRT_VALUE *sub_value;
+      value= myx_grt_list_new(content_type, NULL);
 
-      if (child->type == XML_ELEMENT_NODE && strcmp(child->name, "value")==0)
+      value->value.l->content_struct_name= g_strdup(cstruct_name);
+
+      child= node->children;
+      while (child)
       {
-        sub_value= unserialize_from_xml(child, objects_by_id);
-        if (sub_value)
+        MYX_GRT_VALUE *sub_value;
+
+        if (child->type == XML_ELEMENT_NODE && strcmp(child->name, "value")==0)
         {
-          myx_grt_list_add_item(value, sub_value);
-          myx_grt_value_release(sub_value);
-        }
-        else
-        {
-          myx_grt_value_release(value);
-          value= NULL;
-          break;
+          sub_value= unserialize_from_xml(child, objects_by_id);
+          if (sub_value)
+          {
+            myx_grt_list_item_add(value, sub_value);
+            myx_grt_value_release(sub_value);
+          }
+          else
+          {
+            myx_grt_value_release(value);
+            value= NULL;
+            break;
+          }
         }
+
+        child= child->next;
       }
 
-      child= child->next;
+      xmlFree(ctype);
+      xmlFree(cstruct_name);
     }
-
-    xmlFree(ctype);
-    xmlFree(cstruct_name);
   }
   xmlFree(node_type);
 
@@ -659,6 +849,123 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
 }
 
 
+
+static xmlNodePtr serialize_to_xml_global_object(const char *objectPath, xmlNodePtr parent, MYX_GRT_VALUE *value,
+                                   GHashTable *saved_ids)
+{
+  //unsigned int i;
+  char buffer[100];
+  //const char *id;
+  xmlNodePtr node;
+
+  switch (value->type)
+  {
+  case MYX_INT_VALUE:
+    g_snprintf(buffer, sizeof(buffer), "%i", value->value.i);
+    node= xmlNewTextChild(parent, NULL, "value", buffer);
+    
+    xmlNewProp(node, "type", "int");
+    xmlNewProp(node, "globalObjectPath", objectPath);
+    break;
+  case MYX_REAL_VALUE:
+    g_snprintf(buffer, sizeof(buffer), "%f", value->value.r);
+    node= xmlNewTextChild(parent, NULL, "value", buffer);
+
+    xmlNewProp(node, "type", "real");
+
+    xmlNewProp(node, "global-object-path", objectPath);
+    break;
+  case MYX_STRING_VALUE:
+    node= xmlNewTextChild(parent, NULL, "value", value->value.s);
+
+    xmlNewProp(node, "type", "string");
+    xmlNewProp(node, "global-object-path", objectPath);
+    break;
+  case MYX_LIST_VALUE:
+    node= xmlNewTextChild(parent, NULL, "value", NULL);
+    xmlNewProp(node, "type", "list");
+    xmlNewProp(node, "content-type", myx_get_value_type_as_string(value->value.l->content_type));
+
+    if (value->value.l->content_struct_name)
+      xmlNewProp(node, "content-struct-name", value->value.l->content_struct_name);
+
+    /*for (i= 0; i < value->value.l->items_num; i++)
+    {
+      g_return_val_if_fail((value->value.l->items[i]->type == value->value.l->content_type) ||
+         (value->value.l->content_type == MYX_ANY_VALUE), NULL);
+      serialize_to_xml(node, value->value.l->items[i], saved_ids);
+    }*/
+    xmlNewProp(node, "global-object-path", objectPath);
+    break;
+  case MYX_DICT_VALUE:
+    node= xmlNewTextChild(parent, NULL, "value", NULL);
+    xmlNewProp(node, "type", "dict");
+
+    if ((value->value.d->struct_name) && (value->value.d->struct_name[0]))
+      xmlNewProp(node, "struct-name", value->value.d->struct_name);
+
+    if (value->value.d->content_type != MYX_ANY_VALUE)
+      xmlNewProp(node, "content-type", myx_get_value_type_as_string(value->value.d->content_type));
+
+    if ((value->value.d->content_struct_name) && (value->value.d->content_struct_name[0]))
+      xmlNewProp(node, "content-struct-name", value->value.d->content_struct_name);
+
+    /*if ((id= myx_grt_dict_id_item_as_string(value)))
+    {
+      if (g_hash_table_lookup(saved_ids, id))
+      {
+        // the object is already in the XML tree, just leave a link to it
+        xmlNewProp(node, "link", id);
+
+        return node;
+      }
+      else
+        // object is not yet in XML tree. put it in and record the ID
+        g_hash_table_insert(saved_ids, (gpointer)id, "X");      
+    }
+
+    for (i= 0; i < value->value.d->items_num; i++)
+    {
+      xmlNodePtr child;
+      
+      child= serialize_to_xml(node, value->value.d->items[i].value, saved_ids);
+      xmlNewProp(child, "key", value->value.d->items[i].key);
+    }*/
+    xmlNewProp(node, "global-object-path", objectPath);
+    break;
+  case MYX_ANY_VALUE:
+    break;
+  }
+  return node;
+}
+
+static MYX_GRT_VALUE *unserialize_from_xml_global_object(MYX_GRT *grt, xmlNodePtr node, GHashTable *objects_by_id)
+{
+  xmlChar *node_type= xmlGetProp(node, "type");
+  MYX_GRT_VALUE *value= NULL;
+
+  if (!node_type)
+  {
+    g_warning("Node '%s' in xml doesn't have a type property", node->name);
+    return NULL;
+  }
+
+  // check if we have a global object
+  if ((strcmp(node_type, "dict")==0) || (strcmp(node_type, "list")==0))
+  {
+    xmlChar *global_object_path= xmlGetProp(node, "global-object-path");
+
+    if (global_object_path)
+    {
+      value= myx_grt_value_get_by_path(myx_grt_get_root(grt), global_object_path);
+      xmlFree(global_object_path);
+      return value;
+    }
+  }
+
+  return unserialize_from_xml(node, objects_by_id);
+}
+
 /**
  ****************************************************************************
  * @brief Convert a GRT value into a XML string
@@ -751,6 +1058,99 @@ MYX_GRT_VALUE *myx_grt_value_from_xml(const char *str, size_t size)
   return value;
 }
 
+/**
+ ****************************************************************************
+ * @brief Parse a XML representation of a GRT value that might be a global object.
+ *
+ * Parses a XML string and rebuilds the GRT value that corresponds to it.
+ *
+ * @param grt the GRT environment
+ * @param str the string in XML format containing a serialized GRT value
+ * @param size length of the string
+ *
+ * @return The value corresponding to the passed in string or NULL if there's
+ *   an error.
+ * 
+ * @see myx_grt_value_to_xml
+ *****************************************************************************/
+MYX_GRT_VALUE *myx_grt_value_from_xml_global_object(MYX_GRT *grt, const char *str, size_t size)
+{
+  xmlDocPtr doc= xmlParseMemory(str, (int)size);
+  xmlNodePtr root;
+  MYX_GRT_VALUE *value;
+
+  if (!doc)
+  {
+    g_warning("Could not parse XML data");
+    return NULL;
+  }
+
+  root= xmlDocGetRootElement(doc);
+
+  //check if we have a <data> rootnode
+  if (root && (xmlStrcmp(root->name, "data")==0))
+    root= root->children;
+
+  //skip all nodes that are no <value> nodes
+  while (root && xmlStrcmp(root->name, "value")!=0) 
+    root= root->next;
+
+  if (root)
+  {
+    GHashTable *objects_by_id= g_hash_table_new(g_str_hash, g_str_equal);
+    value= unserialize_from_xml_global_object(grt, root, objects_by_id);
+    g_hash_table_destroy(objects_by_id);
+  }
+  else
+    value= NULL;
+
+  xmlFreeDoc(doc);
+
+  return value;
+}
+
+/**
+ ****************************************************************************
+ * @brief Convert a GRT value into a XML global object string
+ *
+ * Produces a XML representation of the GRT value as a global object
+ * 
+ * @param value a GRT value
+ *
+ * @return String containing the value as a XML or NULL if there's an error.
+ * 
+ * @see myx_grt_value_to_xml
+ *****************************************************************************/
+char *myx_grt_value_to_xml_global_object(const char *objectPath, MYX_GRT_VALUE *value)
+{
+  xmlDocPtr doc;
+  xmlNodePtr root;
+  xmlChar *buffer= NULL;
+  GHashTable *saved_ids;
+  int size;
+  
+  if (value)
+  {
+    doc= xmlNewDoc("1.0");
+    doc->children= root= xmlNewDocRawNode(doc, NULL, "data", NULL);
+    
+    saved_ids= g_hash_table_new(g_str_hash, g_str_equal);
+    
+    serialize_to_xml_global_object(objectPath, root, value, saved_ids);
+    
+    g_hash_table_destroy(saved_ids);
+
+    xmlDocDumpFormatMemory(doc, &buffer, &size, 1);
+
+    xmlFreeDoc(doc);
+
+    return (char*)buffer;
+  }
+  else
+    return NULL;
+}
+
+
 /** 
  ****************************************************************************
  * @brief Add a message to the GRT message queue.
@@ -801,7 +1201,7 @@ void myx_grt_add_msg(MYX_GRT *grt, int msg_type, const char *message, MYX_STRING
  * @return Fetched messages or NULL, if there's none. The returned struct
  * must be freed with myx_grt_free_msgs()
  *****************************************************************************/
-MYX_GRT_MSGS* myx_grt_get_msgs(MYX_GRT *grt, unsigned int count)
+MYX_GRT_MSGS * myx_grt_get_msgs(MYX_GRT *grt, unsigned int count)
 {
   //unsigned int i;
   MYX_GRT_MSGS *msgs;
@@ -845,7 +1245,7 @@ MYX_GRT_MSGS* myx_grt_get_msgs(MYX_GRT *grt, unsigned int count)
  ****************************************************************************
  * @brief 
  *
- * @param msgs  messages to be freed
+ * @param msgs messages to be freed
  *****************************************************************************/
 void myx_grt_free_msgs(MYX_GRT_MSGS *msgs)
 {
@@ -861,3 +1261,142 @@ void myx_grt_free_msgs(MYX_GRT_MSGS *msgs)
   g_free(msgs);
 }
 
+/** 
+ ****************************************************************************
+ * @brief Clears the current cache of _id strings in the root value
+ *
+ * @param grt the GRT environment
+ *****************************************************************************/
+void myx_grt_reference_cache_clear(MYX_GRT *grt)
+{
+  // free current cache
+  g_hash_table_destroy(grt->reference_cache);
+
+  // reallocate cache
+  grt->reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
+                                                g_free, NULL);
+}
+
+
+static void grt_rescan_dict(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *object_path)
+{
+  unsigned int i;
+  const char *struct_name= myx_grt_dict_struct_get_name(dict);
+
+  // if this dict inherits from GrtObject, cache its _id if it not empty
+  // and set the object path
+  if (struct_name && myx_grt_struct_check_inherits_from(grt, struct_name, "GrtObject"))
+  {
+    const char *_id= myx_grt_dict_item_get_as_string(dict, "_id");
+
+    if (_id)
+    {
+      dict->value.d->grt= grt;
+      dict->value.d->object_path= g_strdup(object_path);
+      g_hash_table_insert(grt->reference_cache, g_strdup(_id), dict);
+    }
+  }
+
+  for (i= 0; i<myx_grt_dict_item_count(dict); i++)
+  {
+    const char *key= myx_grt_dict_item_key_by_index(dict, i);
+    MYX_GRT_VALUE *value= myx_grt_dict_item_get_value(dict, key);
+
+    if (myx_grt_value_get_type(value) == MYX_DICT_VALUE) 
+    {
+      char *new_path= g_strdup(object_path);
+      new_path= str_g_append(new_path, "/");
+      new_path= str_g_append(new_path, key);
+
+      grt_rescan_dict(grt, value, new_path);
+
+      g_free(new_path);
+    }
+    else
+      if ( (myx_grt_value_get_type(value) == MYX_LIST_VALUE) && 
+        (myx_grt_list_content_get_type(value) == MYX_DICT_VALUE) )
+      {
+        unsigned int j;
+
+        for (j= 0; j<myx_grt_list_item_count(value); j++)
+        {
+          char *new_path= g_strdup_printf("%s/%s/%d", object_path, key, j);
+
+          grt_rescan_dict(grt, myx_grt_list_item_get(value, j), new_path);
+
+          g_free(new_path);
+        }
+      }
+  }
+}
+
+/** 
+ ****************************************************************************
+ * @brief Rescans the root value to cache all _id strings
+ *
+ * @param grt the GRT environment
+ *****************************************************************************/
+void myx_grt_reference_cache_rescan(MYX_GRT *grt)
+{
+  // start scan
+  grt_rescan_dict(grt, myx_grt_get_root(grt), "");
+}
+
+/** 
+ ****************************************************************************
+ * @brief Performs a reference cache lookup for a specific _id
+ *
+ * @param grt the GRT environment
+ * @param ref_id the _id that is looked up
+ *
+ * @return returns a value if it is found in the cache or NULL
+ *****************************************************************************/
+MYX_GRT_VALUE * myx_grt_reference_cache_lookup(MYX_GRT *grt, const char *ref_id)
+{
+  if(ref_id)
+  {
+    // try to find the value in the cache
+    MYX_GRT_VALUE *value= g_hash_table_lookup(grt->reference_cache, ref_id);
+
+    if (value)
+      return value;
+    else
+    {
+      // if it is not found, rescan and search again
+      myx_grt_reference_cache_rescan(grt);
+
+      return g_hash_table_lookup(grt->reference_cache, ref_id);
+    }
+  }
+  else
+    return NULL;
+}
+
+
+char * myx_grt_get_guid()
+{
+#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+  GUID guid;
+  WCHAR guid_wstr[50];
+  char guid_str[200];
+
+  int len;
+
+  CoCreateGuid(&guid);
+  len= StringFromGUID2(&guid, (LPOLESTR) guid_wstr, 50);
+
+  //Covert GUID from WideChar to utf8
+  WideCharToMultiByte(CP_UTF8, 0, (LPCWSTR) guid_wstr, len,
+    (LPSTR) guid_str, 200, NULL, NULL);
+
+  return g_strdup(guid_str);
+#else
+  {
+    uuid_t gid;
+    char buffer[40];
+    uuid_generate_time(gid);
+    uuid_unparse(gid, buffer);
+    return g_strdup(buffer);
+  }
+#endif
+}
