@@ -1,6 +1,5 @@
 #import "MTableEditor.h"
 #import "MAccessoryScrollView.h"
-#import "MTextImageCell.h"
 #import "NSView_extras.h"
 #import "mxUtils.h"
 #import "myxutil.h"
@@ -437,7 +436,7 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   else if (aTableView == colFlagsList)
   {
     int r= [columnList selectedRow];
-    if (!_table || r < 0 || !_table->columns[r].datatype_pointer)
+    if (!_table || r < 0 || !_table->columns || !_table->columns[r].datatype_pointer)
       return 0;
     return _table->columns[r].datatype_pointer->flags_num;
   }
@@ -735,7 +734,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   NSTableView *table= [aNotification object];
   int row= [table selectedRow];
-  BOOL enabled;
+  BOOL enabled= NO;
 
   if (table == columnList)
   {  
@@ -809,26 +808,38 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   MYX_LIB_ERROR error;
 
   if (_table)
-    myx_dbm_free_table_data(_table);
-
-  _table= myx_dbm_retrieve_table_data(_mysql,_data_types,[_catalogName UTF8String],
-                                      [_schemaName UTF8String],[_tableName UTF8String],
-                                      &error);
-  if (!_table)
   {
-    if (error == MYX_SQL_ERROR)
-      MXRunAlertPanelWithMySQLError(@"Error", 
-                                    [NSString stringWithFormat:@"Could not retrieve data for table '%@'.", _tableName],
-                                    _mysql);
-    else
-      MXRunAlertPanelWithError(@"Error", 
-                               [NSString stringWithFormat:@"Could not retrieve data for table '%@'.", _tableName],
-                               error);
-    return;
+    myx_dbm_free_table_data(_table);
+    _table= NULL;
   }
-  
+
+  if (_tableName)
+  {
+    _table= myx_dbm_retrieve_table_data(_mysql,_data_types,[_catalogName UTF8String],
+                                        [_schemaName UTF8String],[_tableName UTF8String],
+                                        &error);
+    if (!_table)
+    {
+      if (error == MYX_SQL_ERROR)
+        MXRunAlertPanelWithMySQLError(@"Error", 
+                                      [NSString stringWithFormat:@"Could not retrieve data for table '%@'.", _tableName],
+                                      _mysql);
+      else
+        MXRunAlertPanelWithError(@"Error", 
+                                 [NSString stringWithFormat:@"Could not retrieve data for table '%@'.", _tableName],
+                                 error);
+      return;
+    }
+  }
+  else
+  {
+    _table= g_new0(MYX_DBM_TABLE_DATA, 1);
+    
+    _table->charset= g_strdup("utf8");
+  }
+
   // Column Stuff
-  [tableNameText setStringValue:_tableName];
+  [tableNameText setStringValue:_tableName ? : @"new_table"];
   [commentText setStringValue:NSStr(_table->comment)];
   
   [self showColumn:0];
@@ -850,7 +861,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   [fkList reloadData];
   
   // Engine Stuff
-  [[engineMatrix cellWithTag:_table->table_engine] setState:NSOnState];
+  [[engineMatrix cellWithTag:10+_table->table_engine] setState:NSOnState];
   [self refreshForTableType];
   
   // Charset Stuff
@@ -1095,17 +1106,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   }
   else if (sender == engineMatrix)
   {
-    int i;
-    for (i= 0; i < 7; i++)
-    {
-#warning this thing is wrong
-      if ([[engineMatrix cellWithTag:i] state] == NSOnState)
-      {
-        NSLog(@"=== %i %@",i,[[engineMatrix cellWithTag:i] title]);
-        _table->table_engine= i;
-        break;
-      }
-    }
+    _table->table_engine= [[engineMatrix selectedCell] tag] - 10;
     [self refreshForTableType];
   }
 }
@@ -1120,7 +1121,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     
     [self fillCollationsFor:charset popup:colCollationPop];
     
-    if (strcmp(_table->columns[column].charset,charset)!=0)
+    if (strcmp3(_table->columns[column].charset,charset)!=0)
     {
       g_free(_table->columns[column].charset);
       _table->columns[column].charset= g_strdup(charset);
@@ -1287,27 +1288,30 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   NSButton *button;
   MAccessoryScrollView *sv= (MAccessoryScrollView*)[tview enclosingScrollView];
   float size;
-  float fsize= [NSFont smallSystemFontSize];
 
-  size= [NSScroller scrollerWidthForControlSize:NSMiniControlSize];
+//  size= [NSScroller scrollerWidthForControlSize:NSMiniControlSize];
+  size= 16;
 
   button= [[[NSButton alloc] initWithFrame:NSMakeRect(0,0,size,size)] autorelease];
-  [button setBezelStyle: NSShadowlessSquareBezelStyle];
-  [button setTitle:@"+"];
+  [button setButtonType: NSMomentaryChangeButton];
+  [button setBordered:NO];
+  [[button cell] setImagePosition:NSImageOnly];
+  [button setImage: MXGetImageFromBundle([NSBundle bundleForClass:[self class]], @"mini_add.png")];
+  [button setAlternateImage: MXGetImageFromBundle([NSBundle bundleForClass:[self class]], @"mini_add_pressed.png")];
+
   [button setTarget:self];
   [button setAction:addSel];
-  [[button cell] setFont:[NSFont boldSystemFontOfSize:fsize]];
-  [[button cell] setImagePosition:NSNoImage];
   [sv addSubview:button];
   [sv addAccessoryView:button];
   
   button= [[[NSButton alloc] initWithFrame:NSMakeRect(0,0,size,size)] autorelease];
-  [button setBezelStyle: NSShadowlessSquareBezelStyle];
-  [button setTitle:@"-"];
+  [button setButtonType: NSMomentaryChangeButton];
+  [button setBordered:NO];
+  [[button cell] setImagePosition:NSImageOnly];
+  [button setImage: MXGetImageFromBundle([NSBundle bundleForClass:[self class]], @"mini_del.png")];
+  [button setAlternateImage: MXGetImageFromBundle([NSBundle bundleForClass:[self class]], @"mini_del_pressed.png")];
   [button setTarget:self];
   [button setAction:delSel];
-  [[button cell] setFont:[NSFont boldSystemFontOfSize:fsize]];
-  [[button cell] setImagePosition:NSNoImage];
   [sv addSubview:button];
   [sv addAccessoryView:button];
 }
@@ -1344,13 +1348,6 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   _spatialIcon= [[self loadImage:@"datatype_spatial"] retain];
   _stringIcon= [[self loadImage:@"datatype_string"] retain];
   _userdefIcon= [[self loadImage:@"datatype_userdefined"] retain];
-
-  {
-    MTextImageCell *cell= [[[MTextImageCell alloc] init] autorelease];
-    [cell setEditable:YES];
-    [cell setFont:[[[columnList tableColumnWithIdentifier:@"name"] dataCell] font]];
-    [[columnList tableColumnWithIdentifier:@"name"] setDataCell: cell];
-  }
 }
 
 
@@ -1394,7 +1391,10 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   int c, s;
   MYX_SCHEMA_TABLES *my_schema= 0;
   
-  _tableName= [table retain];
+  if (table)
+    _tableName= [table retain];
+  else
+    _tableName= nil;
   _catalogName= [catalog retain];
   _schemaName= [schema retain];
 
@@ -1469,6 +1469,12 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 }
 
 
+- (void)showNewTableForCatalog:(NSString*)catalog
+                        schema:(NSString*)schema
+{
+  [self showTable:nil catalog:catalog schema:schema];
+}
+
 
 - (BOOL)confirmScript:(NSString*)script
 {
