@@ -9,5 +9,11 @@
 #import <Cocoa/Cocoa.h>
 #include "myx_public_interface.h"
 
+NSString *MXGetErrorString(MYX_LIB_ERROR error);
+
 void MXRunAlertPanelWithError(NSString *title, NSString *message, MYX_LIB_ERROR error);
 void MXRunAlertPanelWithMySQLError(NSString *title, NSString *message, MYSQL *mysql);
+
+NSImage *MXGetImageFromBundle(NSBundle *bundle, NSString *image);
+
+void MXExpandOutline(NSOutlineView *outline, BOOL expandChildren);
\ No newline at end of file
