@@ -18,9 +18,8 @@
 #define myx_qb_library_h
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
-#define __LCC__
+#include <winsock2.h>
 #endif
-
 #include "mysql.h"
 #include "mysql_version.h"
 
