@@ -92,8 +92,8 @@ bool MGResultSetModel::iter_next_vfunc(GtkTreeIter* iter)
 
     iter->user_data= (gpointer)((int)iter->user_data+1);
 
-    if ((unsigned)iter->user_data > _resultset->rows_num)
-      g_message("QEWQWE!!!");
+//    if ((unsigned)iter->user_data > _resultset->rows_num)
+//      g_message("QEWQWE!!!");
 
     return (int)iter->user_data < (int)_resultset->rows_num+_new_rows ? true : false;
   }
