@@ -55,11 +55,10 @@ bool MQResultSetView::PopupTreeView::on_button_press_event(GdkEventButton *ev)
     Gtk::TreeViewColumn *column;
     int cx, cy;
     
-    get_path_at_pos((gint)ev->x, (gint)ev->y,
+    if (get_path_at_pos((gint)ev->x, (gint)ev->y,
                     path, column,
-                    cx, cy);
-
-    set_cursor(path, *column, false);
+                    cx, cy))
+      set_cursor(path, *column, false);
   }
 
   if (_popup)
@@ -760,7 +759,7 @@ void MQResultSetView::update_menu()
     myg_menu_set_sensitive(_edit_menu, "load_field", false);
     
     bool flag;
-    if (_model)
+    if (_model && _tree->get_selection()->get_selected())
       flag= true;
     else
       flag= false;
@@ -1142,7 +1141,7 @@ void MQResultSetView::save_cell_change(MGBlobEditor *editor,
   char *cpath= (char*)((Glib::Object*)editor)->get_data("path");
   Gtk::TreePath path(cpath);
   gpointer data;
-  guint size;
+  gsize size;
 
   editor->get_data(data, size);
   _model->set_value(_model->get_iter(path), column*2, data, size);
